﻿// chapter3-1.cpp : このファイルには 'main' 関数が含まれています。プログラム実行の開始と終了がそこで行われます。
//
#include <tchar.h>
#include <stdio.h>
#include "MIDIData.h"
#pragma comment( lib, "MIDIdata.lib" )

// プログラムのスイッチ：ウィンナーワルツ風は１を通常の等時は０を指定
#define _vienna_valtz 0

// 出力するMIDIファイル
#if _vienna_valtz 
const wchar_t* outfilename = L"vienner-valtz.mid";
#else
const wchar_t* outfilename = L"equal-valtz.mid";
#endif

// 音符の定義
#define 	Onpu2  240		// 2分音符の長さ
#define 	Onpu4  120		// 4分音符の長さ
#define 	Onpu8  60		// 8分音符の長さ
#define 	Onpu16  30		// 16分音符の長さ
#define 	FutenOnpu4  180	// 付点4分音符の長さ
#define 	FutenOnpu8  90	// 8分音符の長さ

// 楽器の定義
#define		Violin			48
#define		Viola			48
#define		Cello			48
#define		Bass			48
#define		Snare			38	// Standard Snare

#define 	Velocity 		100	// 強さ
#define 	Velocity1 		70	// 1拍目
#define 	Velocity2 		65	// 2
#define 	Velocity3 		60	// 3

// チャネル、時刻、長さの各変数
int ch1 = 0;			// チャネル1 Cb
int ch2 = 1;			// チャネル2 Vc
int ch3 = 2;			// チャネル3 Va
int ch4 = 3;			// チャネル4 Vn2
int ch5 = 4;			// チャネル5 Vn1
int ch6 = 9;			// チャネル9 スネアドラム

int time1 = 0;			// 時刻 (伴奏群)
int time2 = 0;			// 時刻（メロディ）

// ドレミ
#define 	C1 	36	// ベース用の低音C
#define 	F1 	41	// ベース用の低音F
#define 	C2 	48	// ド
#define 	F2 	53	// ファ
#define 	A2 	57	// ラ
#define 	B2 	59	// ラ
#define 	C3 	60	// ド
#define 	D3 	62	// レ
#define 	E3 	64	// ミ
#define 	F3 	65	// ファ
#define 	G3 	67	// ソ
#define 	A3 	69	// ラ
#define 	B3 	71	// シ
#define 	C4 	72	// ド

#define bar_num 12 	// 小節数

int main()
{
	MIDIData* pMIDIData;
	MIDITrack* pMIDITrack;
	MIDIEvent* pMIDIEvent;

	// MIDIデータの生成(フォーマット0,トラック数1,TPQNベース,120)
	pMIDIData = MIDIData_Create(MIDIDATA_FORMAT0, 1, MIDIDATA_TPQNBASE, 120);
	if (pMIDIData == NULL) {
		printf("MIDIData作成エラー");
		return -1;
	}

	// 最初のトラックへのポインタ
	pMIDITrack = MIDIData_GetFirstTrack(pMIDIData);
	// 各種設定
	MIDITrack_InsertTrackName(pMIDITrack, 0, _T("sample4-1a")); 	// タイトル
	MIDITrack_InsertTempo(pMIDITrack, 0, 60000000 / 180);		// 速度

	// 楽器の指定
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch1, Bass);
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch2, Cello);
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch3, Viola);
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch4, Violin);
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch5, Violin);
	MIDITrack_InsertProgramChange(pMIDITrack, 0, ch6, 0);	// percussion

	// 音符の書き出し
#if _vienna_valtz
	// ウィンナーワルツ風
	for (int bar = 0; bar < bar_num; bar++) {
		MIDITrack_InsertNote(pMIDITrack, time1, ch1, F1, Velocity1, Onpu4); // Bass
		MIDITrack_InsertNote(pMIDITrack, time1, ch2, F2, Velocity1, Onpu4); // Cello
		// 計測された1拍目の長さの比を掛ける
		time1 += Onpu4 * 0.74;	
		MIDITrack_InsertNote(pMIDITrack, time1, ch3, C3, Velocity2, Onpu8); // Viola
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, A2, Velocity2, Onpu8); // Violin2
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, F3, Velocity2, Onpu8); // Violin2
		time1 += Onpu4 * 1.23;
		MIDITrack_InsertNote(pMIDITrack, time1, ch3, C3, Velocity3, Onpu8); // Viola
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, A2, Velocity2, Onpu8); // Violin2
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, F3, Velocity2, Onpu8); // Violin2
		time1 += Onpu4 * 1.03;
		// Tempo 四分音符=120 の場合、durationをトータルで360にすると、3/4拍子になる。

	}
#else	// 機械的な３拍子の場合
	for (int bar = 0; bar < bar_num; bar++) {
		MIDITrack_InsertNote(pMIDITrack, time1, ch1, F1, Velocity1, Onpu4); // Bass
		MIDITrack_InsertNote(pMIDITrack, time1, ch2, F2, Velocity1, Onpu4); // Cello
		time1 += Onpu4;	// 音符は8分音符でも拍は4分音符分進めるとスタッカートのようになる
		MIDITrack_InsertNote(pMIDITrack, time1, ch3, C3, Velocity1, Onpu8); // Viola
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, A2, Velocity1, Onpu8); // Violin2
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, F3, Velocity1, Onpu8); // Violin2
		time1 += Onpu4;
		MIDITrack_InsertNote(pMIDITrack, time1, ch3, C3, Velocity1, Onpu8); // Viola
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, A2, Velocity1, Onpu8); // Violin2
		MIDITrack_InsertNote(pMIDITrack, time1, ch4, F3, Velocity1, Onpu8); // Violin2
		//MIDITrack_InsertNote(pMIDITrack, time1, ch5, Snare, Velocity1, Onpu16); // Snare
		time1 += Onpu4;
		// Tempo 笙ｩ=120 の場合、durationをトータルで360にすると、3/4拍子になる。
	}
#endif

	// メロディ
	time2 = 120 * 3 * 3 + 120 * 2 + 60; // ４小節目３拍裏まで休み

	MIDITrack_InsertNote(pMIDITrack, time2, ch5, G3 + 1, Velocity, Onpu8); // Violin1 G#
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B2, Velocity, Onpu8); // Violin1 B
	time2 += Onpu8;
	// 5小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu2 * Onpu8); // Violin1
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu2 * Onpu8); // Violin1
	time2 += Onpu2 + Onpu8;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, G3 + 1, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B3, Velocity, Onpu8);
	time2 += Onpu8;
	// 6小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu2 + Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu2 + Onpu8);
	time2 += Onpu2 + Onpu8;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, G3 + 1, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B3, Velocity, Onpu8);
	time2 += Onpu8;
	// 7小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu8);
	time2 += Onpu2;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C4, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, E3, Velocity, Onpu8);
	time2 += Onpu4;
	// 8小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C4, Velocity, Onpu2 + Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, E3, Velocity, Onpu2 + Onpu8);
	time2 += Onpu2 + Onpu8;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B3 - 1, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, D3, Velocity, Onpu8);
	time2 += Onpu8;
	// 9小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B3 - 1, Velocity, Onpu2 + Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, D3, Velocity, Onpu2 + Onpu8);
	time2 += Onpu2 + Onpu8 + 15;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu8);
	time2 += Onpu8;
	// 10小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu2 + Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu2 + Onpu8);
	time2 += Onpu2 + Onpu8;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, F3 + 1, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A2, Velocity, Onpu8);
	time2 += Onpu8;
	// 11小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, A3, Velocity, Onpu4);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, C3, Velocity, Onpu4);
	time2 += Onpu2;
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, G3, Velocity, Onpu8);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B2 - 1, Velocity, Onpu8);
	time2 += Onpu4;
	// 12小節目
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, G3, Velocity, Onpu2);
	MIDITrack_InsertNote(pMIDITrack, time2, ch5, B2 - 1, Velocity, Onpu2);
	time2 += Onpu2;


	// トラックの終端　トータルの時間も指定する
	MIDITrack_InsertEndofTrack(pMIDITrack, time2);
	//  MIDIデータとして保存
	MIDIData_SaveAsSMF(pMIDIData, outfilename);
	//	MIDIData_SaveAsSMF(pMIDIData, _T("3byoushi.mid"));
		// データの削除
	MIDIData_Delete(pMIDIData);
	pMIDIData = NULL;

	return 0;
}
// プログラムの実行: Ctrl + F5 または [デバッグ] > [デバッグなしで開始] メニュー
// プログラムのデバッグ: F5 または [デバッグ] > [デバッグの開始] メニュー
